#include <iostream>
#include <list>
using namespace std;

#include "../syssoft_linux.cpp"

#include "fifo.cpp"

// ***********************************************************************
// Sender
//
// Returns the total number of bytes sent
//
// ***********************************************************************

double Sender ( Fifo *p, size_t message_size, double time_to_run ) {
	Moment start, stop;
	char message[message_size];
	char c = 0;
	unsigned int n_messages = 0;

	start.now();
	do {
		for (int i=0; i<message_size; i++) message[i] = c++;
		p->send(message,message_size);
		n_messages++;
		stop.now();
	} while ( stop-start < time_to_run );

	// Send the final message
	for (int i=0; i<message_size; i++) message[i] = (char) 42;
	p->send(message,message_size);

	return (1.0 + n_messages) * message_size;
}

// ***********************************************************************
// Main
// ***********************************************************************

int main ( int ac, char ** av, char **env ) {

	if (ac != 3) {
		cerr << "usage: " << av[0] << " <message_size> <time to run>" << endl;
		exit(0);
	}

	size_t message_size = atol(av[1]);
	double runtime = atof(av[2]);

	Moment start, stop;

	char *args[4];
	args[0] = "./fifo_perf_receiver_o2";
	args[1] = "the_named_pipe";
	args[2] = av[1];
	args[3] = 0;

	Process receiver(args[0],args,env);

	Fifo p("the_named_pipe",true);

	start.now();
	double n_bytes = Sender(&p,message_size,runtime);
	receiver.wait();
	stop.now();

	cout << message_size << "\t" << 1000.0 * n_bytes / (stop-start) / 1024.0 / 1024.0 << endl;
}



